@@ -62,7 +62,8 @@ class NoneObject(object):
     Instantiate with the reason for the error.
     """
     def __init__(self, reason = '', strict = False):
-        debug.debug("None object instantiated: " + reason, 2)
+        if not hasattr(sys, "frozen"):
+            debug.debug("None object instantiated: " + reason, 2)
         self.reason = reason
         self.strict = strict
         if strict:
@@ -440,7 +441,10 @@ class NativeType(BaseObject, NumericProxyMixIn):
         if not data:
             return NoneObject("Unable to read {0} bytes from {1}".format(self.size(), self.obj_offset))
 
-        (val,) = struct.unpack(self.format_string, data)
+        try:
+            (val,) = struct.unpack(self.format_string, data)
+        except struct.error:
+            return NoneObject("struct.error {0} bytes from {1}".format(self.size(), self.obj_offset))
 
         # Ensure that integer NativeTypes are converted to longs
         # to avoid integer boundaries when doing __rand__ proxying
