@@ -62,9 +62,6 @@ class ImpScan(common.AbstractWindowsCommand):
         config.add_option('SIZE', short_option = 's', default = None,
                           help = 'Size of memory to scan',
                           action = 'store', type = 'int')
-        ## FIXME. ImpScan currently does not work on wow64 processes. 
-        ## Add an option to override the profile's memory_model and 
-        ## allow 32bit disasm on x64 operating systems. 
 
         self.forwarded_imports = {
             "RtlGetLastWin32Error" : "kernel32.dll!GetLastError",
@@ -117,7 +114,7 @@ class ImpScan(common.AbstractWindowsCommand):
                 op.mnemonic == "JMP"))
 
     def _vicinity_scan(self, addr_space, calls_imported,
-                apis, base_address, data_len, forward):
+                apis, base_address, data_len, is_wow64 = False, forward = True):
         """Scan forward from the lowest IAT entry found or
         backward from the highest IAT entry found. We do this 
         because not every imported function will be called 
@@ -129,7 +126,8 @@ class ImpScan(common.AbstractWindowsCommand):
         @param apis: dictionary of exported functions in the AS 
         @param base_address: memory base address 
         @param data_len: size in bytes to check from base_address
-        @param forwared: the direction for the vicinity scan
+        @param is_wow64: True if its a Wow64 process
+        @param forward: the direction for the vicinity scan
         """
 
         sortedlist = calls_imported.keys()
@@ -138,7 +136,12 @@ class ImpScan(common.AbstractWindowsCommand):
         if not sortedlist:
             return
 
-        size_of_address = addr_space.profile.get_obj_size("address")
+        if is_wow64:
+            addr_type = "int"
+        else:
+            addr_type = "address"
+
+        size_of_address = addr_space.profile.get_obj_size(addr_type)
 
         if forward:
             start_addr = sortedlist[0]
@@ -157,7 +160,7 @@ class ImpScan(common.AbstractWindowsCommand):
             else:
                 next_addr = start_addr - (i * size_of_address)
 
-            call_dest = obj.Object("address", offset = next_addr,
+            call_dest = obj.Object(addr_type, offset = next_addr,
                             vm = addr_space).v()
 
             if (not call_dest or
@@ -190,7 +193,7 @@ class ImpScan(common.AbstractWindowsCommand):
         else:
             return mod_name, func_name
 
-    def call_scan(self, addr_space, base_address, data):
+    def call_scan(self, addr_space, base_address, data, is_wow64 = False):
         """Disassemble a block of data and yield possible 
         calls to imported functions. We're looking for 
         instructions such as these:
@@ -211,26 +214,28 @@ class ImpScan(common.AbstractWindowsCommand):
 
         @param addr_space: an AS to scan with
         @param base_address: memory base address
-        @param data: buffer of data found at base_address 
+        @param data: buffer of data found at base_address
+        @param is_wow64: True if its a Wow64 process
         """
 
         end_address = base_address + len(data)
 
         memory_model = addr_space.profile.metadata.get('memory_model', '32bit')
 
-        if memory_model == '32bit':
+        if memory_model == '32bit' or is_wow64:
             mode = distorm3.Decode32Bits
+            addr_type = "int"
         else:
             mode = distorm3.Decode64Bits
+            addr_type = "address"
 
         for op in distorm3.DecomposeGenerator(base_address, data, mode):
-
             if not op.valid:
                 continue
 
             iat_loc = None
 
-            if memory_model == '32bit':
+            if memory_model == '32bit' or is_wow64:
                 if (self._call_or_unc_jmp(op) and
                         op.operands[0].type == 'AbsoluteMemoryAddress'):
                     iat_loc = (op.operands[0].disp) & 0xffffffff
@@ -246,7 +251,7 @@ class ImpScan(common.AbstractWindowsCommand):
                 continue
 
             # This is the address being called 
-            call_dest = obj.Object("address", offset = iat_loc,
+            call_dest = obj.Object(addr_type, offset = iat_loc,
                             vm = addr_space)
 
             if call_dest == None:
@@ -305,6 +310,9 @@ class ImpScan(common.AbstractWindowsCommand):
             data = kernel_space.zread(base_address, size_to_read)
             apis = self.enum_apis(all_mods)
             addr_space = kernel_space
+
+            is_wow64 = False
+
         else:
             # In process mode, we find the process by PID
             task = None
@@ -347,6 +355,8 @@ class ImpScan(common.AbstractWindowsCommand):
                 base_address = all_mods[0].DllBase
                 size_to_read = all_mods[0].SizeOfImage
 
+            is_wow64 = task.IsWow64
+
             data = task_space.zread(base_address, size_to_read)
             apis = self.enum_apis(all_mods)
             addr_space = task_space
@@ -354,19 +364,17 @@ class ImpScan(common.AbstractWindowsCommand):
         # This is a dictionary of confirmed API calls.
         calls_imported = dict(
                 (iat, call)
-                for (_, iat, call) in self.call_scan(addr_space, base_address, data)
+                for (_, iat, call) in self.call_scan(addr_space, base_address, data, is_wow64)
                 if call in apis
                 )
 
         # Scan forward 
         self._vicinity_scan(addr_space,
-                calls_imported, apis, base_address, len(data),
-                forward = True)
+                calls_imported, apis, base_address, len(data), is_wow64, forward = True)
 
         # Scan reverse 
         self._vicinity_scan(addr_space,
-                calls_imported, apis, base_address, len(data),
-                forward = False)
+                calls_imported, apis, base_address, len(data), is_wow64, forward = False)
 
         for iat, call in sorted(calls_imported.items()):
             yield iat, call, apis[call][0], apis[call][1]
