@@ -26,8 +26,11 @@
 """
 
 # Information for this script taken heavily from File System Forensic Analysis by Brian Carrier
+from volatility import renderers
 
 import volatility.plugins.common as common
+from volatility.renderers.basic import Address
+import volatility.plugins.overlays.basic as basic
 import volatility.scan as scan
 import volatility.utils as utils
 import volatility.addrspace as addrspace
@@ -37,6 +40,22 @@ import struct
 import binascii
 import os
 import volatility.poolscan as poolscan
+import sys
+reload(sys)
+sys.setdefaultencoding('utf8')
+
+class UnicodeString(basic.String):
+    def __str__(self):
+        result = self.obj_vm.zread(self.obj_offset, self.length).split("\x00\x00")[0].decode("utf16", "ignore")
+        if not result:
+            result = ""
+        return result
+
+    def v(self):
+        result = self.obj_vm.zread(self.obj_offset, self.length).split("\x00\x00")[0].decode("utf16", "ignore")
+        if not result:
+            return obj.NoneObject("Cannot read string length {0} at {1:#x}".format(self.length, self.obj_offset))
+        return result
 
 ATTRIBUTE_TYPE_ID = {
     0x10:"STANDARD_INFORMATION",
@@ -118,7 +137,7 @@ MFT_PATHS_FULL = {}
 
 class MFT_FILE_RECORD(obj.CType):
     def remove_unprintable(self, str):
-        return ''.join([c for c in str if (ord(c) > 31 or ord(c) == 9) and ord(c) <= 126])
+        return str.encode("utf8", "ignore")
 
     def add_path(self, fileinfo):
         # it doesn't really make sense to add regular files to parent directory,
@@ -128,7 +147,7 @@ class MFT_FILE_RECORD(obj.CType):
             return
         # otherwise keep a record of the directory that we've found
         cur = MFT_PATHS_FULL.get(int(self.RecordNumber), None)
-        if cur == None or cur["filename"].find("~") != -1 and fileinfo.is_valid():
+        if (cur == None or fileinfo.Namespace != 2) and fileinfo.is_valid():
             temp = {}
             temp["ParentDirectory"] = fileinfo.ParentDirectory
             temp["filename"] = self.remove_unprintable(fileinfo.get_name())
@@ -169,7 +188,7 @@ class MFT_FILE_RECORD(obj.CType):
     def get_mft_type(self):
         return "{0}{1}".format("In Use & " if self.is_inuse() else "",
                "Directory" if self.is_directory() else "File")
-        
+
     def parse_attributes(self, mft_buff, check = True, entrysize = 1024):
         next_attr = self.ResidentAttributes
         end = mft_buff.find("\xff\xff\xff\xff")
@@ -224,30 +243,38 @@ class MFT_FILE_RECORD(obj.CType):
             elif attr == "DATA":
                 if self.obj_vm._config.DEBUGOUT:
                     print "Found $DATA"
-                if next_attr.Header.NameOffset > 0 and next_attr.Header.NameLength > 0:
-                    adsname = ""
-                    if next_attr != None and next_attr.Header != None and next_attr.Header.NameOffset and next_attr.Header.NameLength:
-                        nameloc = next_attr.obj_offset + next_attr.Header.NameOffset
-                        adsname = obj.Object("NullString", vm = self.obj_vm, offset = nameloc, length = next_attr.Header.NameLength * 2)
-                        if adsname != None and adsname.strip() != "" and dataseen:
-                            attr += " ADS Name: {0}".format(adsname.strip())
-                dataseen = True
-                if next_attr.ContentSize == 0:
-                    next_off = next_attr.obj_offset + self.obj_vm.profile.get_obj_size("RESIDENT_ATTRIBUTE")
-                    next_attr = self.advance_one(next_off, mft_buff, end)
-                    attributes.append((attr, ""))
+                try:
+                    if next_attr.Header and next_attr.Header.NameOffset > 0 and next_attr.Header.NameLength > 0:
+                        adsname = ""
+                        if next_attr != None and next_attr.Header != None and next_attr.Header.NameOffset and next_attr.Header.NameLength:
+                            nameloc = next_attr.obj_offset + next_attr.Header.NameOffset
+                            adsname = obj.Object("UnicodeString", vm = self.obj_vm, offset = nameloc, length = next_attr.Header.NameLength * 2)
+                            if adsname != None and adsname.strip() != "" and dataseen:
+                                attr += " ADS Name: {0}".format(adsname.strip())
+                    dataseen = True
+                except struct.error:
+                    next_attr = None
+                    continue
+                try:
+                    if next_attr.ContentSize == 0:
+                        next_off = next_attr.obj_offset + self.obj_vm.profile.get_obj_size("RESIDENT_ATTRIBUTE")
+                        next_attr = self.advance_one(next_off, mft_buff, end)
+                        attributes.append((attr, ""))
+                        continue
+                    start = next_attr.obj_offset + next_attr.ContentOffset
+                    theend = min(start + next_attr.ContentSize, end)
+                except struct.error:
+                    next_attr = None
                     continue
-                start = next_attr.obj_offset + next_attr.ContentOffset
-                theend = min(start + next_attr.ContentSize, end)
                 if next_attr.Header.NonResidentFlag == 1:
-                    thedata = "" 
+                    thedata = ""
                 else:
                     try:
                         contents = mft_buff[start:theend]
                     except TypeError:
                         next_attr = None
                         continue
-                    thedata = contents 
+                    thedata = contents
                 attributes.append((attr, thedata))
                 next_off = theend
                 if next_off == start:
@@ -271,7 +298,7 @@ class MFT_FILE_RECORD(obj.CType):
     def advance_one(self, next_off, mft_buff, end):
         item = None
         attr = None
-        cursor = 0 
+        cursor = 0
 
         if next_off == None:
             return None
@@ -317,11 +344,30 @@ class STANDARD_INFORMATION(obj.CType):
     # XXX need a better check than this
     # we return valid if we have _any_ timestamp other than Null
     def is_valid(self):
-        return obj.CType.is_valid(self) and (self.ModifiedTime.v() != 0 or self.MFTAlteredTime.v() != 0 or \
-                self.FileAccessedTime.v() != 0 or self.CreationTime.v() != 0) 
+        try:
+            modified = self.ModifiedTime.v()
+        except struct.error:
+            modified = 0 
+        try:
+            mftaltered = self.MFTAlteredTime.v()
+        except struct.error:
+            mftaltered = 0 
+        try:
+            creation = self.CreationTime.v()
+        except struct.error:
+            creation = 0 
+        try:
+            accessed = self.FileAccessedTime.v()
+        except struct.error:
+            accessed = 0
+        return obj.CType.is_valid(self) and (modified != 0 or mftaltered != 0 or \
+                accessed != 0 or creation != 0)
 
     def get_type_short(self):
-        if self.Flags == None:
+        try:
+            if self.Flags == None:
+                return "?"
+        except struct.error:
             return "?"
         type = ""
         for i, j in sorted(SHORT_STANDARD_INFO_FLAGS.items()):
@@ -333,8 +379,12 @@ class STANDARD_INFORMATION(obj.CType):
 
 
     def get_type(self):
-        if self.Flags == None:
+        try:
+            if self.Flags == None:
+                return "Unknown Type"
+        except struct.error:
             return "Unknown Type"
+
         type = None
         for i in VERBOSE_STANDARD_INFO_FLAGS:
             if (i & self.Flags) == i:
@@ -343,7 +393,7 @@ class STANDARD_INFORMATION(obj.CType):
                 else:
                     type += " & " + VERBOSE_STANDARD_INFO_FLAGS[i]
         if type == None:
-            type = "Unknown Type " 
+            type = "Unknown Type "
         return type
 
     def get_header(self):
@@ -353,55 +403,103 @@ class STANDARD_INFORMATION(obj.CType):
                 ("Access Date", "30"),
                 ("Type", ""),
                ]
-   
+
     def __str__(self):
-        return "{0:20} {1:30} {2:30} {3:30} {4}".format(str(self.CreationTime),
-            str(self.ModifiedTime),
-            str(self.MFTAlteredTime),
-            str(self.FileAccessedTime),
-            self.get_type())
+        bufferas = addrspace.BufferAddressSpace(self.obj_vm._config, data = "\x00\x00\x00\x00\x00\x00\x00\x00")
+        nulltime = obj.Object("WinTimeStamp", vm = bufferas, offset = 0, is_utc = True)
+        try:
+            modified = str(self.ModifiedTime)
+        except struct.error:
+            modified = nulltime
+        try:
+            mftaltered = str(self.MFTAlteredTime)
+        except struct.error:
+            mftaltered = nulltime
+        try:
+            creation = str(self.CreationTime)
+        except struct.error:
+            creation = nulltime
+        try:
+            accessed = str(self.FileAccessedTime)
+        except struct.error:
+            accessed = nulltime
+
+        return "{0:20} {1:30} {2:30} {3:30} {4}".format(creation, modified, mftaltered, accessed, self.get_type())
 
     def body(self, path, record_num, size, offset):
         if path.strip() == "" or path == None:
-            # if the path is null we just try to get the filename 
+            # if the path is null we just try to get the filename
             # from our dictionary and print the body file output
-            record = MFT_PATHS_FULL.get(int(record_num), {}) 
+            record = MFT_PATHS_FULL.get(int(record_num), {})
             path = "(Possible non-base entry, extra $SI or invalid $FN)"
             if record != {}:
-                # we include with the found filename a note that this may be a 
+                # we include with the found filename a note that this may be a
                 # non-base entry.  the analyst can investigate these types of records
-                # on his/her own by comparing record numbers in output or examining the 
+                # on his/her own by comparing record numbers in output or examining the
                 # given physical offset in memory for example
                 path = "{0} {1}".format(record["filename"], path)
 
+        try:
+            modified = self.ModifiedTime.v()
+        except struct.error:
+            modified = 0
+        try:
+            mftaltered = self.MFTAlteredTime.v()
+        except struct.error:
+            mftaltered = 0
+        try:
+            creation = self.CreationTime.v()
+        except struct.error:
+            creation = 0
+        try:
+            accessed = self.FileAccessedTime.v()
+        except struct.error:
+            accessed = 0
+
         return "[{9}MFT STD_INFO] {0} (Offset: 0x{1:x})|{2}|{3}|0|0|{4}|{5}|{6}|{7}|{8}".format(
             path,
             offset,
             record_num,
             self.get_type_short(),
             size,
-            self.FileAccessedTime.v(),
-            self.ModifiedTime.v(),
-            self.MFTAlteredTime.v(),
-            self.CreationTime.v(),
+            accessed,
+            modified,
+            mftaltered,
+            creation,
             self.obj_vm._config.MACHINE)
 
 class FILE_NAME(STANDARD_INFORMATION):
     def remove_unprintable(self, str):
-        return ''.join([c for c in str if (ord(c) > 31 or ord(c) == 9) and ord(c) <= 126])
+        return str.encode("utf8", "ignore")
 
     # XXX need a better check than this
     # we return valid if we have _any_ timestamp other than Null
     # filename must also be a non-empty string
     def is_valid(self):
-        return obj.CType.is_valid(self) and (self.ModifiedTime.v() != 0 or self.MFTAlteredTime.v() != 0 or \
-                self.FileAccessedTime.v() != 0 or self.CreationTime.v() != 0) and \
-                self.remove_unprintable(self.get_name()) != ""
+        try:
+            modified = self.ModifiedTime.v()
+        except struct.error:
+            modified = 0
+        try:
+            mftaltered = self.MFTAlteredTime.v()
+        except struct.error:
+            mftaltered = 0
+        try:
+            creation = self.CreationTime.v()
+        except struct.error:
+            creation = 0
+        try:
+            accessed = self.FileAccessedTime.v()
+        except struct.error:
+            accessed = 0
+        return obj.CType.is_valid(self) and (modified != 0 or mftaltered != 0 or \
+                accessed != 0 or creation != 0) #and \
+                #self.remove_unprintable(self.get_name()) != ""
 
     def get_name(self):
         if self.NameLength == None or self.NameLength == 0:
             return ""
-        return "{0}".format(str(self.Name).replace("\x00", ""))
+        return self.remove_unprintable(self.Name)
 
     def get_header(self):
         return [("Creation", "30"),
@@ -412,33 +510,83 @@ class FILE_NAME(STANDARD_INFORMATION):
                ]
 
     def __str__(self):
-        return "{0:20} {1:30} {2:30} {3:30} {4}".format(str(self.CreationTime),
-            str(self.ModifiedTime),
-            str(self.MFTAlteredTime),
-            str(self.FileAccessedTime),
+        bufferas = addrspace.BufferAddressSpace(self.obj_vm._config, data = "\x00\x00\x00\x00\x00\x00\x00\x00")
+        nulltime = obj.Object("WinTimeStamp", vm = bufferas, offset = 0, is_utc = True)
+        try:
+            modified = str(self.ModifiedTime)
+        except struct.error:
+            modified = nulltime
+        try:
+            mftaltered = str(self.MFTAlteredTime)
+        except struct.error:
+            mftaltered = nulltime
+        try:
+            creation = str(self.CreationTime)
+        except struct.error:
+            creation = nulltime
+        try:
+            accessed = str(self.FileAccessedTime)
+        except struct.error:
+            accessed = nulltime
+
+        return "{0:20} {1:30} {2:30} {3:30} {4}".format(creation, modified, mftaltered, accessed,
             self.remove_unprintable(self.get_name()))
 
     def get_full(self, full):
+        bufferas = addrspace.BufferAddressSpace(self.obj_vm._config, data = "\x00\x00\x00\x00\x00\x00\x00\x00")
+        nulltime = obj.Object("WinTimeStamp", vm = bufferas, offset = 0, is_utc = True)
         try:
-            return "{0:20} {1:30} {2:30} {3:30} {4}".format(str(self.CreationTime),
-                str(self.ModifiedTime),
-                str(self.MFTAlteredTime),
-                str(self.FileAccessedTime),
+            modified = str(self.ModifiedTime)
+        except struct.error:
+            modified = nulltime
+        try:
+            mftaltered = str(self.MFTAlteredTime)
+        except struct.error:
+            mftaltered = nulltime
+        try:
+            creation = str(self.CreationTime)
+        except struct.error:
+            creation = nulltime
+        try:
+            accessed = str(self.FileAccessedTime)
+        except struct.error:
+            accessed = nulltime
+        try:
+            return "{0:20} {1:30} {2:30} {3:30} {4}".format(creation,
+                modified,
+                mftaltered,
+                accessed,
                 self.remove_unprintable(full))
         except struct.error:
             return None
 
     def body(self, path, record_num, size, offset):
+        try:
+            modified = self.ModifiedTime.v()
+        except struct.error:
+            modified = 0
+        try:
+            mftaltered = self.MFTAlteredTime.v()
+        except struct.error:
+            mftaltered = 0
+        try:
+            creation = self.CreationTime.v()
+        except struct.error:
+            creation = 0
+        try:
+            accessed = self.FileAccessedTime.v()
+        except struct.error:
+            accessed = 0
         return "[{9}MFT FILE_NAME] {0} (Offset: 0x{1:x})|{2}|{3}|0|0|{4}|{5}|{6}|{7}|{8}".format(
             path,
             offset,
             record_num,
             self.get_type_short(),
             size,
-            self.FileAccessedTime.v(),
-            self.ModifiedTime.v(),
-            self.MFTAlteredTime.v(),
-            self.CreationTime.v(),
+            accessed,
+            modified,
+            mftaltered,
+            creation,
             self.obj_vm._config.MACHINE)
 
 class OBJECT_ID(obj.CType):
@@ -480,19 +628,19 @@ MFT_types = {
      }],
 
     'ATTRIBUTE_HEADER': [ 0x10, {
-        'Type': [0x0, ['int']],   
+        'Type': [0x0, ['int']],
         'Length': [0x4, ['int']],
         'NonResidentFlag': [0x8, ['unsigned char']],
         'NameLength': [0x9, ['unsigned char']],
         'NameOffset': [0xa, ['unsigned short']],
         'Flags': [0xc, ['unsigned short']],
         'AttributeID': [0xe, ['unsigned short']],
-    }], 
+    }],
 
     'RESIDENT_ATTRIBUTE': [0x16, {
         'Header': [0x0, ['ATTRIBUTE_HEADER']],
         'ContentSize': [0x10, ['unsigned int']], #relative to the beginning of the attribute
-        'ContentOffset': [0x14, ['unsigned short']], 
+        'ContentOffset': [0x14, ['unsigned short']],
         'STDInfo': lambda x : obj.Object("STANDARD_INFORMATION", offset = x.obj_offset + x.ContentOffset, vm = x.obj_vm),
         'FileName': lambda x : obj.Object("FILE_NAME", offset = x.obj_offset + x.ContentOffset, vm = x.obj_vm),
         'ObjectID': lambda x : obj.Object("OBJECT_ID", offset = x.obj_offset + x.ContentOffset, vm = x.obj_vm),
@@ -516,13 +664,13 @@ MFT_types = {
         'EaCount': [0x4, ['int']],
         'EaUnpackedLength': [0x8, ['long']],
     }],
- 
+
     'EA': [None, {
         'NextEntryOffset': [0x0, ['unsigned long long']],
         'Flags': [0x8, ['unsigned char']],
         'EaNameLength': [0x9, ['unsigned char']],
         'EaValueLength': [0xa, ['unsigned short']],
-        'EaName': [0xc, ['String', dict(length = lambda x: x.EaNameLength)]], 
+        'EaName': [0xc, ['String', dict(length = lambda x: x.EaNameLength)]],
         'EaValue': lambda x: obj.Object("Array", offset = x.obj_offset + len(x.EaName), count = x.EaValueLength, vm = x.obj_vm,
                                         target = obj.Curry(obj.Object, "unsigned char")),
     }],
@@ -555,7 +703,7 @@ MFT_types = {
         'ReparseValue': [0x3c, ['unsigned int']],
         'NameLength': [0x40, ['unsigned char']],
         'Namespace': [0x41, ['unsigned char']],
-        'Name': [0x42, ['NullString', dict(length = lambda x: x.NameLength * 2)]],
+        'Name': [0x42, ['UnicodeString', dict(length = lambda x: x.NameLength * 2)]],
     }],
 
     'ATTRIBUTE_LIST': [0x19, {
@@ -636,6 +784,7 @@ class MFTTYPES(obj.ProfileModification):
     conditions = {'os': lambda x: x == 'windows'}
     def modification(self, profile):
         profile.object_classes.update({
+            'UnicodeString':UnicodeString,
             'MFT_FILE_RECORD':MFT_FILE_RECORD,
             'FILE_NAME':FILE_NAME,
             'STANDARD_INFORMATION':STANDARD_INFORMATION,
@@ -646,12 +795,12 @@ class MFTTYPES(obj.ProfileModification):
 
 
 class MFTScanner(scan.BaseScanner):
-    checks = [ ] 
+    checks = [ ]
 
     def __init__(self, needles = None):
         self.needles = needles
         self.checks = [ ("MultiStringFinderCheck", {'needles':needles})]
-        scan.BaseScanner.__init__(self) 
+        scan.BaseScanner.__init__(self)
 
     def scan(self, address_space, offset = 0, maxlen = None):
         for offset in scan.BaseScanner.scan(self, address_space, offset, maxlen):
@@ -662,7 +811,7 @@ class MFTParser(common.AbstractWindowsCommand):
     """ Scans for and parses potential MFT entries """
     def __init__(self, config, *args, **kwargs):
         common.AbstractWindowsCommand.__init__(self, config, *args, **kwargs)
-        config.add_option("OFFSET", short_option = "o", default = None, 
+        config.add_option("OFFSET", short_option = "o", default = None,
                           help = "Physical offset for MFT Entries (comma delimited)")
         config.add_option('NOCHECK', short_option = 'N', default = False,
                           help = 'Only all entries including w/null timestamps',
@@ -690,7 +839,7 @@ class MFTParser(common.AbstractWindowsCommand):
                 mft_buff = address_space.read(offset, self._config.ENTRYSIZE)
                 bufferas = addrspace.BufferAddressSpace(self._config, data = mft_buff)
                 mft_entry = obj.Object('MFT_FILE_RECORD', vm = bufferas, offset = 0)
-                offsets.append((offset, mft_entry))
+                offsets.append((offset, mft_entry, mft_buff))
         else:
             scanner = poolscan.MultiPoolScanner(needles = ['FILE', 'BAAD'])
             print "Scanning for MFT entries and building directory, this can take a while"
@@ -698,21 +847,27 @@ class MFTParser(common.AbstractWindowsCommand):
             for _, offset in scanner.scan(address_space):
                 mft_buff = address_space.read(offset, self._config.ENTRYSIZE)
                 bufferas = addrspace.BufferAddressSpace(self._config, data = mft_buff)
-                mft_entry = obj.Object('MFT_FILE_RECORD', vm = bufferas,
-                               offset = 0)
-                temp = mft_entry.advance_one(mft_entry.ResidentAttributes.STDInfo.obj_offset + mft_entry.ResidentAttributes.ContentSize, mft_buff, self._config.ENTRYSIZE)
                 name = ""
-                if temp != None:
+                try:
+                    mft_entry = obj.Object('MFT_FILE_RECORD', vm = bufferas,
+                               offset = 0)
+                    temp = mft_entry.advance_one(mft_entry.ResidentAttributes.STDInfo.obj_offset + mft_entry.ResidentAttributes.ContentSize, mft_buff, self._config.ENTRYSIZE)
+                    if temp == None:
+                        continue
                     mft_entry.add_path(temp.FileName)
                     name = temp.FileName.get_name()
+                except struct.error:
+                    if self._config.DEBUGOUT:
+                        print "Problem entry at offset:", hex(offset)
+                    continue
+    
                 if (int(mft_entry.RecordNumber), name) in seen:
                     continue
                 else:
                     seen.append((int(mft_entry.RecordNumber), name))
-                offsets.append((offset, mft_entry))
+                offsets.append((offset, mft_entry, mft_buff))
 
-        for offset, mft_entry in offsets:
-            mft_buff = address_space.read(offset, self._config.ENTRYSIZE)
+        for offset, mft_entry, mft_buff in offsets:
             if self._config.DEBUGOUT:
                 print "Processing MFT Entry at offset:", hex(offset)
             attributes = mft_entry.parse_attributes(mft_buff, not self._config.NOCHECK, self._config.ENTRYSIZE)
@@ -769,6 +924,81 @@ class MFTParser(common.AbstractWindowsCommand):
                 # here we have a lone $SI in an MFT entry with no valid $FN.  This is most likely a non-base entry
                 outfd.write("0|{0}\n".format(si.body("", mft_entry.RecordNumber, -1, offset)))
 
+    def unified_output(self, data):
+        return renderers.TreeGrid([("MFT Offset", Address),
+                            ("Attribute", str),
+                            ("Record", int),
+                            ("Link count", int),
+                            ("Type", str),
+                            ("Creation", str),
+                            ("Modified", str),
+                            ("MFT Altered", str),
+                            ("Access Date", str),
+                            ("Value", str)], self.generator(data))
+
+    def generator(self, data):
+        bufferas = addrspace.BufferAddressSpace(self._config, data = "\x00\x00\x00\x00\x00\x00\x00\x00")
+        nulltime = obj.Object("WinTimeStamp", vm = bufferas, offset = 0, is_utc = True)
+        for offset, mft_entry, attributes in data:
+            if not len(attributes):
+                continue
+            datnum = 0
+            for a, i in attributes:
+                if i == None:
+                    attrdata = ["Invalid (" + a + ")", "", "", "", "", ""]
+                elif a.startswith("STANDARD_INFORMATION"):
+                    try:
+                        modified = str(i.ModifiedTime)
+                    except struct.error:
+                        modified = nulltime
+                    try:
+                        mftaltered = str(i.MFTAlteredTime)
+                    except struct.error:
+                        mftaltered = nulltime
+                    try:
+                        creation = str(i.CreationTime)
+                    except struct.error:
+                        creation = nulltime
+                    try:
+                        accessed = str(i.FileAccessedTime)
+                    except struct.error:
+                        accessed = nulltime
+                    attrdata = [a, creation,
+                                modified,
+                                mftaltered,
+                                accessed,
+                                i.get_type()]
+                elif a.startswith("FILE_NAME"):
+                    try:
+                        modified = str(i.ModifiedTime)
+                    except struct.error:
+                        modified = nulltime
+                    try:
+                        mftaltered = str(i.MFTAlteredTime)
+                    except struct.error:
+                        mftaltered = nulltime
+                    try:
+                        creation = str(i.CreationTime)
+                    except struct.error:
+                        creation = nulltime
+                    try:
+                        accessed = str(i.FileAccessedTime)
+                    except struct.error:
+                        accessed = nulltime
+                    attrdata = [a, creation,
+                                modified,
+                                mftaltered,
+                                accessed,
+                                i.remove_unprintable(i.get_name())]
+                else:
+                    attrdata = [a, "", "", "", "", ""]
+
+                yield (0, [Address(offset),
+                           str(mft_entry.get_mft_type()),
+                           int(mft_entry.RecordNumber),
+                           int(mft_entry.LinkCount)] + attrdata)
+
+
     def render_text(self, outfd, data):
         if self._config.DUMP_DIR != None and not os.path.isdir(self._config.DUMP_DIR):
             debug.error(self._config.DUMP_DIR + " is not a directory")
@@ -778,7 +1008,7 @@ class MFTParser(common.AbstractWindowsCommand):
                 continue
             outfd.write("{0}\n".format(border))
             outfd.write("MFT entry found at offset 0x{0:x}\n".format(offset))
-            outfd.write("Attribute: {0}\n".format(mft_entry.get_mft_type())) 
+            outfd.write("Attribute: {0}\n".format(mft_entry.get_mft_type()))
             outfd.write("Record Number: {0}\n".format(mft_entry.RecordNumber))
             outfd.write("Link count: {0}\n".format(mft_entry.LinkCount))
             outfd.write("\n")
