@@ -13,8 +13,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
-#
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 
 # ELF64 format: http://downloads.openwatcom.org/ftp/devel/docs/elf-64-gen.pdf
 
@@ -308,10 +307,18 @@ class elf_hdr(elf):
 
         tname = "elf_phdr"
         
+        if self.e_phoff < 0 or self.e_phoff > 1000000:
+            return
+
         # the buffer of headers
         arr_start = self.obj_offset + self.e_phoff
 
-        for i in range(self.e_phnum):
+        if self.e_phnum > 128:
+            phnum = 128
+        else:
+            phnum = self.e_phnum
+
+        for i in range(phnum):
             # use the real size
             idx = i * rtsize
 
@@ -325,19 +332,20 @@ class elf_hdr(elf):
 
         tname = "elf_shdr"
        
-        # the buffer of headers
-        arr_start = self.obj_offset + self.e_shoff
+        if self.e_shoff < 1:
+            arr_start = -1
+        else:
+            # the buffer of headers
+            arr_start = self.obj_offset + self.e_shoff
 
         return (arr_start, rtsize)
 
-    def section_header(self, idx):
-        (arr_start, rtsize) = self._section_headers()
-        idx = idx * rtsize
-        shdr = obj.Object("elf_shdr", offset = arr_start + idx, vm = self.obj_vm, parent = self)
-        return shdr
-    
     def section_headers(self):
         (arr_start, rtsize) = self._section_headers()
+
+        if arr_start == -1:
+            return
+
         for i in range(self.e_shnum):
             # use the real size
             idx = i * rtsize
@@ -556,6 +564,9 @@ class elf_phdr(elf):
     def __init__(self, theType, offset, vm, name = None, **kwargs):
         elf.__init__(self, 0, "elf32_phdr", "elf64_phdr", theType, offset, vm, name, **kwargs)    
 
+    def is_valid(self):
+        return self.p_filesz > 0 and self.p_memsz > 0
+
     @property
     def p_vaddr(self):
         ret = self.__getattr__("p_vaddr")
@@ -568,7 +579,6 @@ class elf_phdr(elf):
     def dynamic_sections(self):
         # sanity check
         if str(self.p_type) != 'PT_DYNAMIC':
-            print "failed sanity check"
             return
 
         rtname = self._get_typename("dyn")
@@ -681,16 +691,27 @@ class elf_link_map(elf):
         tname = "elf_link_map"
         return obj.Object(tname, offset = naddr, vm = self.obj_vm, parent = self)
 
-    def __iter__(self):
-        cur = self
+    def _walk_map_list(self, access_func):
+        seen = []
+        cur  = self
         while cur:
-            yield cur
-            cur = cur.l_next
+            if cur.obj_offset in seen:
+                break
 
-        cur = self
-        while cur:
             yield cur
-            cur = cur.l_prev
+
+            seen.append(cur.obj_offset)
+
+            # check for signs of infinite looping
+            if len(seen) > 1024:
+                break
+
+            cur = access_func(cur)
+
+    def __iter__(self):        
+        for member in [lambda x: x.l_next, lambda x: x.l_prev]:
+            for mapinfo in self._walk_map_list(member):
+                yield mapinfo
 
 class elf32_link_map(obj.CType):
     def __init__(self, theType, offset, vm, name = None, **kwargs):
